@@ -40,12 +40,6 @@ class NoMediaRepoVolumeGroupFound(AbstractMediaException):
                 "media repository.")
 
 
-class ManagementPartitionNotFoundException(nex.NovaException):
-    """Couldn't find exactly one management partition on the system."""
-    msg_fmt = _("Expected to find exactly one management partition; found "
-                "%(count)d.")
-
-
 class NoDiskDiscoveryException(nex.NovaException):
     """Failed to discover any disk."""
     msg_fmt = _("Having scanned SCSI bus %(bus)x on the management partition, "
@@ -115,3 +109,22 @@ class VolumeDetachFailed(nex.NovaException):
 class VolumePreMigrationFailed(nex.NovaException):
     msg_fmt = _("Unable to perform pre live migration steps on volume (id: "
                 "%(volume_id)s) from virtual machine %(instance_name)s.")
+
+
+class PowerVMAPIFailed(nex.NovaException):
+    msg_fmt = _("PowerVM API Failed to complete for instance=%(inst_name)s."
+                "%(reason)s")
+
+
+class ViosNotAvailable(nex.NovaException):
+    msg_fmt = _("No Virtual I/O Servers are available.  The driver attempted "
+                "to wait for a VIOS to become active for %(wait_time)d "
+                "seconds.  The compute agent is not able to start if no "
+                "Virtual I/O Servers are available.  Please check the RMC "
+                "connectivity between the PowerVM NovaLink and the Virtual "
+                "I/O Servers and then restart the Nova Compute Agent.")
+
+
+class InvalidRebuild(nex.NovaException):
+    msg_fmt = _("Unable to rebuild virtual machine on new host.  Error is "
+                "%(error)s")
