@@ -31,6 +31,7 @@ from pypowervm.tasks import power
 from pypowervm.tasks import vterm
 from pypowervm import util as pvm_util
 from pypowervm.utils import lpar_builder as lpar_bldr
+from pypowervm.utils import transaction as pvm_trans
 from pypowervm.utils import uuid as pvm_uuid
 from pypowervm.utils import validation as vldn
 from pypowervm.wrappers import base_partition as pvm_bp
@@ -53,19 +54,23 @@ POWERVM_TO_NOVA_STATE = {
     pvm_bp.LPARState.MIGRATING_RUNNING: power_state.RUNNING,
     pvm_bp.LPARState.RUNNING: power_state.RUNNING,
     pvm_bp.LPARState.STARTING: power_state.RUNNING,
+    # map open firmware state to active since it can be shut down
+    pvm_bp.LPARState.OPEN_FIRMWARE: power_state.RUNNING,
+    # It is running until it is off.
+    pvm_bp.LPARState.SHUTTING_DOWN: power_state.RUNNING,
+    # It is running until the suspend completes
+    pvm_bp.LPARState.SUSPENDING: power_state.RUNNING,
 
     pvm_bp.LPARState.MIGRATING_NOT_ACTIVE: power_state.SHUTDOWN,
     pvm_bp.LPARState.NOT_ACTIVATED: power_state.SHUTDOWN,
 
+    pvm_bp.LPARState.UNKNOWN: power_state.NOSTATE,
     pvm_bp.LPARState.HARDWARE_DISCOVERY: power_state.NOSTATE,
     pvm_bp.LPARState.NOT_AVAILBLE: power_state.NOSTATE,
-    # map open firmware state to active since it can be shut down
-    pvm_bp.LPARState.OPEN_FIRMWARE: power_state.RUNNING,
-    pvm_bp.LPARState.RESUMING: power_state.NOSTATE,
-    pvm_bp.LPARState.SHUTTING_DOWN: power_state.NOSTATE,
-    pvm_bp.LPARState.SUSPENDING: power_state.NOSTATE,
-    pvm_bp.LPARState.UNKNOWN: power_state.NOSTATE,
 
+    # While resuming, we should be considered suspended still.  Only once
+    # resumed will we be active (which is represented by the RUNNING state)
+    pvm_bp.LPARState.RESUMING: power_state.SUSPENDED,
     pvm_bp.LPARState.SUSPENDED: power_state.SUSPENDED,
 
     pvm_bp.LPARState.ERROR: power_state.CRASHED
@@ -91,10 +96,12 @@ RESUMING_EVENTS = [
     pvm_bp.LPARState.RESUMING,
 ]
 
-POWERVM_STARTABLE_STATE = (pvm_bp.LPARState.NOT_ACTIVATED)
-POWERVM_STOPABLE_STATE = (pvm_bp.LPARState.RUNNING, pvm_bp.LPARState.STARTING,
-                          pvm_bp.LPARState.OPEN_FIRMWARE,
-                          pvm_bp.LPARState.ERROR, pvm_bp.LPARState.RESUMING)
+POWERVM_STARTABLE_STATE = (pvm_bp.LPARState.NOT_ACTIVATED, )
+POWERVM_STOPABLE_STATE = (
+    pvm_bp.LPARState.RUNNING, pvm_bp.LPARState.STARTING,
+    pvm_bp.LPARState.OPEN_FIRMWARE, pvm_bp.LPARState.SHUTTING_DOWN,
+    pvm_bp.LPARState.ERROR, pvm_bp.LPARState.RESUMING,
+    pvm_bp.LPARState.SUSPENDING)
 
 
 def translate_event(pvm_state, pwr_state):
@@ -598,12 +605,17 @@ def rename(adapter, host_uuid, instance, name, entry=None):
         be fetched.
     :returns: The updated LPAR wrapper.
     """
-
     if not entry:
         entry = get_instance_wrapper(adapter, instance, host_uuid)
 
-    entry.name = pvm_util.sanitize_partition_name_for_api(name)
-    return entry.update()
+    hyp_name = pvm_util.sanitize_partition_name_for_api(name)
+
+    @pvm_trans.entry_transaction
+    def _rename(entry):
+        entry.name = hyp_name
+        return entry.update()
+
+    return _rename(entry)
 
 
 def dlt_lpar(adapter, lpar_uuid):
@@ -649,6 +661,10 @@ def power_off(adapter, instance, host_uuid, entry=None, add_parms=None,
         entry = get_instance_wrapper(adapter, instance, host_uuid)
 
     # Get the current state and see if we can stop the VM
+    LOG.debug("Powering off request for instance %(inst)s which is in "
+              "state %(state)s.  Force Immediate Flag: %(force)s.",
+              {'inst': instance.name, 'state': entry.state,
+               'force': force_immediate})
     if entry.state in POWERVM_STOPABLE_STATE:
         # Now stop the lpar
         try:
